#!/usr/bin/env python

from bisect import bisect_left, insort_left

# Time management.

def have_conflict(freebusy, periods, get_conflicts=False):

    """
    Return whether any period in 'freebusy' overlaps with the given 'periods',
    returning a collection of such overlapping periods if 'get_conflicts' is
    set to a true value.
    """

    conflicts = []
    for start, end in periods:
        overlapping = period_overlaps(freebusy, (start, end), get_conflicts)
        if overlapping:
            if get_conflicts:
                conflicts += overlapping
            else:
                return True

    if get_conflicts:
        return conflicts
    else:
        return False

def insert_period(freebusy, period):
    insort_left(freebusy, period)

def remove_period(freebusy, uid):
    i = 0
    while i < len(freebusy):
        t = freebusy[i]
        if len(t) >= 3 and t[2] == uid:
            del freebusy[i]
        else:
            i += 1

def period_overlaps(freebusy, period, get_periods=False):

    """
    Return whether any period in 'freebusy' overlaps with the given 'period',
    returning a collection of overlapping periods if 'get_periods' is set to a
    true value.
    """

    dtstart, dtend = period[:2]
    found = bisect_left(freebusy, (dtstart, dtend, None, None))

    overlapping = []

    # Find earlier overlapping periods.

    i = found

    while i > 0 and freebusy[i - 1][1] > dtstart:
        if get_periods:
            overlapping.insert(0, freebusy[i - 1])
        else:
            return True
        i -= 1

    # Find later overlapping periods.

    i = found

    while i < len(freebusy) and (dtend is None or freebusy[i][0] < dtend):
        if get_periods:
            overlapping.append(freebusy[i])
        else:
            return True
        i += 1

    if get_periods:
        return overlapping
    else:
        return False

# vim: tabstop=4 expandtab shiftwidth=4
